<?php
/*
|--------------------------------------------------------------------------
| chat.php 处理实时聊天
|--------------------------------------------------------------------------
*/
namespace Weixin\Src;

use Dever;
use Weixin\Src\Lib\Core;

class Chat
{
	public $info;
	public $account;

	/**
     * 根据openid获取某个用户的聊天记录
     * 
     * @return mixed
     */
	public function __construct()
	{
		Dever::load('manage/auth.init');
		$id = Dever::input('id');
		$this->info = Dever::load('weixin/usermsg-one', $id);
		if (!$this->info) {
			Dever::alert('错误的信息');
		}
		# 默认头像
		$this->info['default'] = 'http://static.duoshuo.com/images/noavatar_default.png';
		
		# 设置客服的头像的名称
		$this->info['name'] = '客服';
		$this->info['avatar'] = $this->info['default'];
		
		
		if ($this->info) {
			# 分析出该记录是做了什么，说了什么，把说的和做的都分别统计出来
			$this->info['user'] = Dever::load('weixin/user-info', array('where_openid' => $this->info['from']));
			
			if (!$this->info['user']) {
				Dever::alert('错误的用户信息');
			}
		
			if (!$this->info['user']['headimgurl']) {
				$this->info['user']['headimgurl'] = $this->info['default'];
			}
			
			$this->info['account'] = 0;
		}
	}
	
	/**
     * 获取信息
     * 
     * @return mixed
     */
	public function info()
	{
		return $this->info;
	}
	
	/**
     * 获取用户信息
     * 
     * @return mixed
     */
	public function user()
	{
		return $this->info['user'];
	}
	
	/**
     * 发送客服消息
     * 
     * @return mixed
     */
	public function send()
	{
		$content = Dever::input('content');
		if (!$content) {
			Dever::alert('请填写内容');
		}
		# 根据openid 获取用户最新事件
		$info = Dever::load('weixin/usermsg-getNew', array('where_from' => $this->info['from']));
		
		if ($info) {
			Dever::setInput('id', $info['id']);
			Dever::load('weixin/message.send', $info);
		}
	}

	/**
     * 根据openid获取某个用户的聊天记录
     * 
     * @return mixed
     */
	public function usermsg()
	{
		$this->account();

		$data = Dever::load('weixin/usermsg-getAll', array('where_from' => $this->info['from']));
		$result = array();
		foreach ($data as $k => $v) {
			$result[$k] = $this->user_event($v);
			$result[$k]['name'] = $this->info['user']['nickname'];
			$result[$k]['avatar'] = $this->info['user']['headimgurl'];
		}

		return $result;
	}

	/**
     * 获取用户行为
     * 
     * @return mixed
     */
	private function user_event($info)
	{
		$chat = array('text', 'image', 'voice', 'video');

		if (in_array($info['type'], $chat)) {
			$result['type'] = '聊天';
		} else {
			$result['type'] = '事件';
		}

		$result['time'] = date('Y-m-d H:i:s', $info['ctime']);
		$result['content'] = Dever::load('weixin/user.msg', $info);
		
		if (isset($this->account[$info['id']])) {
			$result['child'] = $this->account[$info['id']];
			foreach ($result['child'] as $k => $v) {
				$result['child'][$k]['name'] = $this->info['name'];
				$result['child'][$k]['avatar'] = $this->info['avatar'];
				$result['child'][$k]['time'] = date('Y-m-d H:i:s', $result['child'][$k]['cdate']);
				if ($result['child'][$k]['state'] == 1) {
					$result['child'][$k]['status'] = '发送成功';
				} else {
					$result['child'][$k]['status'] = '发送失败';
				}
				
				$this->info['account']++;
			}
		}

		return $result;
	}

	/**
     * 根据openid获取某个客服的聊天记录
     * 
     * @return mixed
     */
	private function account()
	{
		$this->account = Dever::load('weixin/account_msg-getAll', array('where_openid' => $this->info['from']));
	}
}