<?php
/*
|--------------------------------------------------------------------------
| service.php 业务处理
|--------------------------------------------------------------------------
*/
namespace Weixin\Src;

use Dever;
use Weixin\Src\Lib\Core;

class Service
{
	/**
	 * core
	 *
	 * @var Core
	 */
	private $core;
	
	/**
	 * result
	 *
	 * @var array
	 */
	private $result;
	
	/**
	 * output
	 *
	 * @var string
	 */
	private $output;
	
	/**
     * 构造函数　初始化
     * 
     * @return mixed
     */
	public function __construct()
	{
		$site = Dever::input('site', 1);
		$this->core = new Core($site);
	}
	
	/**
     * 业务推送接口 微信服务器会将所有请求都推送到这里
     * 
     * @return mixed
     */
	public function push()
	{
		# 接入检测时
		//return $this->check();
		
		$this->load();
	}

	/**
     * 微信接入检测接口
     * 
     * @return mixed
     */
	private function check()
	{
		$config = $this->core->config();

        $signature = Dever::input('signature');
        $timestamp = Dever::input('timestamp');
        $nonce = Dever::input('nonce');
        $echostr = Dever::input('echostr');
	        		
		$token = $config['token'];
		$info = array($token, $timestamp, $nonce);
		sort($info, SORT_STRING);
		$info = implode($info);
		$info = sha1($info);
		
		if ($info == $signature && $echostr) {
			echo $echostr;die;
		}
	}
	
	/**
     * 对微信发送的内容做出处理
     * 
     * @return mixed
     */
	public function load()
	{
		$this->check();
		
		$this->output = '';
		
		# 获取微信消息
		$this->get();
		
		# 对消息进行验证，并根据类型得到本平台的配置数据
		$this->response();
		
		# 输出
		$this->output();
	}
	
	/**
     * 输出
     * 
     * @return mixed
     */
	private function output()
	{
		if (!$this->output) {
			# 如果没有匹配到，则直接返回默认回复消息
			$this->msg('msg2');

			$this->output();
		} else {
			if (isset($this->result['id']) && isset($this->result['reply'])) {
				Dever::load('weixin/usermsg-update', array('set_reply' => $this->result['reply'], 'where_id' => $this->result['id']));
			}
			
			echo $this->output;die;
		}
	}
	
	/**
     * 据类型得到本平台的配置数据
     * 
     * @return mixed
     */
	private function response()
	{
		if ($this->output) {
			return;
		}
		
		if ($this->result && isset($this->result['add_type'])) {
			if (isset($this->result['subscribe'])) {
				if ($this->result['subscribe'] == true) {
					# 订阅成功 发送消息
					$this->msg('msg1');
					# 发送消息之后，往用户表更新信息
					$this->user();
					$this->setSubscribe(1);
				}
				if ($this->result['subscribe'] == false) {
					# 取消订阅 不发送消息
					//$this->msg('msg1');
					# 改变当前用户的订阅状态
					$this->setSubscribe(2);
				}
			} elseif (isset($this->result['add_content'][0]['Content'])) {
				$this->match($this->result['add_content'][0]['Content']);
			} else {
				# 如果没有匹配到，则直接返回默认回复消息
				$this->msg('msg2');
			}
		}
	}

	/**
     * 同步当前用户信息
     * 
     * @return mixed
     */
	private function user()
	{
		Dever::daemon('user.sync?openid=' . $this->result['add_from'], 'weixin');
		/*
		Dever::setInput('openid', $this->result['add_from']);

		$data = Dever::load('weixin/data.user_info');
		if($data && isset($data['data']) && isset($data['data']['groupid']))
		{
			# 分组用户数量加+1
			Dever::load('weixin/group-upNum', array('where_groupid' => $data['data']['groupid'], 'set_count' => 1));
		}
		*/
	}

	/**
     * 改变订阅状态
     * 
     * @return mixed
     */
	private function setSubscribe($type = 0)
	{
		Dever::load('weixin/user-upSubscribe', array('where_openid' => $this->result['add_from'], 'set_subscribe' => $type));
	}
	
	/**
     * 默认消息
     * 
     * @return mixed
     */
	private function msg($type)
	{
		if ($type == 'msg2') {
			# 检查最近是否有客服手动接管 就按照24小时计算吧
			$kf = Dever::load('weixin/account_msg-getOne', array('where_openid' => $this->result['add_from']));
			$time = 3600*24;
			if ($kf && $kf['cdate'] - time() > $time) {
				$this->result['reply'] = '客服手动接管,无自动回复,24小时后无客服回复自动解除。';
				
				Dever::load('weixin/usermsg-update', array('set_reply' => $this->result['reply'], 'where_id' => $this->result['id']));
				
				echo '';die;
			}
		}
		
		$info = Dever::load('weixin/config-one', $this->result['add_site']);
		if ($info && $info[$type]) {
			$info['type'] = 1;
			$info['info'] = $info[$type];
			$this->cat($info);
			
			# 记录回复的数据
			$this->reply('默认匹配：' . $info['info']);
		}
	}
	
	/**
     * 消息内容匹配
     * 
     * @return mixed
     */
	private function match($content)
	{
		$keywords = Dever::load('weixin/content-get_2');
		if ($keywords) {
			foreach ($keywords as $k => $v) {
				if ($v['keywords']) {
					$temp = explode("\r\n", $v['keywords']);
					foreach ($temp as $ki => $vi) {
						if (strpos($content, $vi) !== false) {
							$this->cat($v);
							# 记录回复的数据
							$this->reply('关键词匹配：' . $vi);
							return;
						}
					}
				}
			}
		}
		# 如果没有匹配到，则直接返回默认回复消息
		$this->msg('msg2');
	}
	
	/**
     * 拼装回复内容
     * 
     * @return mixed
     */
	private function reply($name)
	{
		$this->result['reply'] = array();
		$this->result['reply']['类型'] = $name;
		$this->result['reply']['内容'] = str_replace("\r\n", '<br />', htmlspecialchars($this->output));
		
		$this->result['reply'] = Dever::table($this->result['reply']);
	}
	
	/**
     * 拼装要发送给微信的内容
     * 
     * @return mixed
     */
    private function cat($data)
    {
		$keywords = $this->keywords($data['type']);
		if (!$keywords) {
			return;
		}
		$config = $this->config($keywords, 'set');
		if (!$config) {
			return;
		}

		$xml = '';
		$total = 1;
		
		if ($data['type'] == 1 && isset($data['text']) && $data['text']) {
			$data['info'] = $data['text'];
		}
		
		# 处理多图文的情况
		if ($data['type'] == 2 && $data['content_2']) {
			$data['content'] = json_decode(base64_decode($data['content_2']), true);

			$total = count($data['content']);
		} else {
			switch ($data['type']) {
				# 语音
				case 4:
					$key = 'mp3';
					break;
				# 视频
				case 5:
					$key = 'mp4';
					break;
				# 图片
				case 6:
					$key = 'pic';
					break;
			}
			
			if (isset($key)) {
				$file = $data[$key];
				$info = Dever::load('weixin/media-file', array('where_file' => $file, 'where_site' => $this->core->site()));
				if ($info && $info['mid']) {
					$data[$key] = $info['mid'];
				}
			}
			
		}

		$xml = $this->createXml($config, $data, $total);
		
		$this->output = $this->xml($xml);
	}
	
	/**
     * 拼装要发送给微信的内容
     * 
     * @return mixed
     */
	private function createXml($config, $data, $total = 1)
	{
		if (is_array($config)) {
			$xml = '';
			foreach ($config as $k => $v) {
				if (!is_array($v)) {
					if (isset($data[$v])) {
						$v = $data[$v];
					} elseif ($v == 'total') {
						$v = $total;
					}
					$xml .= $this->tag($k, $v);
				} elseif (isset($data['type']) && $data['type'] == 2 && $data['content']) {
					$item = '';
					foreach ($data['content'] as $ki => $vi) {
						$item .= $this->createXml($v, $vi);
					}
					
					$xml .= $this->tag($k, $item);
				} else {
					$xml .= $this->tag($k, $this->createXml($v, $data));
				}
			}
		} else {
			$xml = $this->tag($config, $data['info']);
		}
		
		return $xml;
	}

    /**
     * 获取微信发送的内容
     * 
     * @return mixed
     */
    private function get()
    {
		$post = isset($GLOBALS["HTTP_RAW_POST_DATA"]) ? $GLOBALS["HTTP_RAW_POST_DATA"] : false;

/*
		$post = ' <xml>
 <ToUserName><![CDATA[toUser]]></ToUserName>
 <FromUserName><![CDATA[ow29VvzICifpOTq_9EDGT3AhYpqg]]></FromUserName> 
 <CreateTime>1442067297</CreateTime>
 <MsgType><![CDATA[text]]></MsgType>
 <Content><![CDATA[dfd]]></Content>
 <MsgId>1311112322222222</MsgId>
 </xml>';
 
 /*
		$post = '<xml>
<ToUserName><![CDATA[toUser]]></ToUserName>
<FromUserName><![CDATA[FromUser]]></FromUserName>
<CreateTime>123456789</CreateTime>
<MsgType><![CDATA[event]]></MsgType>
<Event><![CDATA[subscribe]]></Event>
</xml>';


		$post = ' <xml>
 <ToUserName><![CDATA[toUser]]></ToUserName>
 <FromUserName><![CDATA[ow29VvzICifpOTq_9EDGT3AhYpqg]]></FromUserName>
 <CreateTime>1442067297</CreateTime>
 <MsgType><![CDATA[image]]></MsgType>
 <PicUrl><![CDATA[http://mmbiz.qpic.cn/mmbiz/OxUBpiaYgpHiaDaiccrsX3MrFEVtjIfEAZk8zoJDZ4gXWWW2CIFic6Dicqndu23wMgA4Icg3yE4SRZg6L5jqanu9kvQ/0]]></PicUrl>
 <MediaId><![CDATA[media_id]]></MediaId>
 <MsgId>111</MsgId>
 </xml>';

 		$post = '<xml>
<ToUserName><![CDATA[toUser]]></ToUserName>
<FromUserName><![CDATA[ow29VvzICifpOTq_9EDGT3AhYpqg]]></FromUserName>
<CreateTime>1442067297</CreateTime>
<MsgType><![CDATA[voice]]></MsgType>
<MediaId><![CDATA[media_id]]></MediaId>
<Format><![CDATA[Format]]></Format>
<MsgId>23123213</MsgId>
</xml>';


		$post = '<xml>
<ToUserName><![CDATA[toUser]]></ToUserName>
<FromUserName><![CDATA[FromUser]]></FromUserName>
<CreateTime>123456789</CreateTime>
<MsgType><![CDATA[event]]></MsgType>
<Event><![CDATA[CLICK]]></Event>
<EventKey><![CDATA[EVENTKEY]]></EventKey>
</xml>';
*/
		if (!empty($post)) {
			libxml_disable_entity_loader(true);
          	$result = (array) simplexml_load_string($post, 'SimpleXMLElement', LIBXML_NOCDATA);

          	$this->handle($result);
		}
    }
    
    /**
     * 对微信的内容做处理，得到最终结果
     * 
     * @return mixed
     */
    private function handle($result)
    {
		$config = array
		(
			'FromUserName' => 'add_from',
			'ToUserName' => 'add_to',
			'MsgType' => 'add_type',
			'MsgId' => 'add_mid',
			'CreateTime' => 'add_ctime',
		);
		
		foreach ($config as $k => $v) {
			$this->result($k, $v, $result);
		}
		
		# 事件处理
		$this->event();
		
		$this->add();
	}
	
	/**
     * 事件处理
     * 
     * @return mixed
     */
	private function event()
	{
		if ($this->result['add_type'] == 'event') {
			$config = $this->result['add_content'][0];
			$type = $config['Event'];
			$this->result['add_type'] = $type;
			$table = '';
			# 取消订阅
			if ($type == 'unsubscribe') {
				$this->result['add_info'] = '取消关注';
				$this->result['subscribe'] = false;
			}
			
			# 二维码扫描
			# 用户未关注时，通过二维码关注并进入场景，需要根据ticket得到二维码
			elseif ($type == 'subscribe' && isset($config['Ticket'])) {
				$this->result['add_info'] = '关注并扫描二维码';
				# 给用户回复
				$this->result['subscribe'] = true;
				$table = 'code';
			}
			# 用户已关注，通过二维码关注并进入场景，需要根据ticket得到二维码
			elseif ($type == 'SCAN' && isset($config['Ticket'])) {
				$this->result['add_info'] = '扫描二维码';
				$table = 'code';
			}
			
			
			# 用户关注
			elseif ($type == 'subscribe') {
				$this->result['add_info'] = '关注';
				# 给用户回复
				$this->result['subscribe'] = true;
			}
			# 用户上报地理位置
			elseif ($type == 'LOCATION') {
				$this->result['add_info'] = '上报地理位置';
			}
			
			# 菜单处理
			# 用户点击菜单
			elseif ($type == 'CLICK') {
				$this->result['add_info'] = '点击菜单';
				$table = 'menu';
			}
			# 用户点击菜单跳转
			elseif ($type == 'VIEW') {
				$this->result['add_info'] = '跳转链接:' . $config['EventKey'];
				//$table = 'menu';
				
				//$this->result['add_info'] = '链接:' . $config['EventKey'];
				//$table = 'menu';
			}
			# 扫码推事件的事件推送
			elseif ($type == 'scancode_push') {
				$this->result['add_info'] = '通过菜单扫码';
				$table = 'menu';
			}
			# 扫码推事件且弹出“消息接收中”提示框的事件推送
			elseif ($type == 'scancode_waitmsg') {
				$this->result['add_info'] = '通过菜单扫码并弹出提示';
				$table = 'menu';
			}
			# 弹出系统拍照发图的事件推送
			elseif ($type == 'pic_sysphoto') {
				$this->result['add_info'] = '通过菜单弹出系统拍照组件';
				$table = 'menu';
			}
			# 弹出拍照或者相册发图的事件推送
			elseif ($type == 'pic_photo_or_album') {
				$this->result['add_info'] = '通过菜单弹出拍照或者相册发图组件';
				$table = 'menu';
			}
			# 弹出微信相册发图器的事件推送
			elseif ($type == 'pic_weixin') {
				$this->result['add_info'] = '通过菜单弹出微信相册发图器';
				$table = 'menu';
			}
			# 弹出地理位置选择器的事件推送
			elseif ($type == 'location_select') {
				$this->result['add_info'] = '通过菜单弹出地理位置选择器';
				$table = 'menu';
			}

			# 群发消息通知
			elseif ($type == 'MASSSENDJOBFINISH') {
				$this->result['add_info'] = '群发消息微信自动回复';
			}

			# 根据eventkey查询当前的设置
			if (isset($config['EventKey']) && $table) {
				$id = $config['EventKey'];
				if (strpos($config['EventKey'], '_') !== false) {
					$temp = explode('_', $config['EventKey']);
					$id = end($temp);
				}
				if ($id > 0) {
					$info = Dever::load('weixin/'.$table.'-one', $id);
					if ($info) {
						$this->result['add_info'] .= ':' . $info['name'];

						if ($info['message'] > 0) {
							$keywords = Dever::load('weixin/content-one', $info['message']);
							if($keywords) $this->cat($keywords);
						}

						# 记录回复的数据
						$this->reply('事件匹配：' . $this->result['add_info']);
					}
				}
			}
		}
	}
	
	/**
     * 入库
     * 
     * @return mixed
     */
	private function add()
	{
		$this->result['add_info'] = is_array($this->result['add_info']) ? implode(',', $this->result['add_info']) : $this->result['add_info'];
		$this->result['add_site'] = $this->core->site();
		
		# 入库　使用mid排重
		$info = array();
		if (isset($this->result['add_mid'])) {
			$info = Dever::load('weixin/usermsg-getOne', array('where_mid' => $this->result['add_mid'], 'where_site' => $this->result['add_site']));
			$this->result['id'] = $info['id'];
		}

		if (!$info) {
			$this->result['id'] = Dever::load('weixin/usermsg-insert', $this->result);

			# 记录点对点聊天记录 暂时不记录了
			/*
			if($this->result['add_type'] == 'text')
			{
				$update['add_state'] = 1;
				$update['add_openid'] = $this->result['add_from'];
				$update['add_usermsg_id'] = $this->result['id'];
				$update['add_account'] = -1;
				$update['add_type'] = $this->result['add_type'];
				$update['add_info'] = Dever::load('weixin/user.msg', $this->result['id']);

				Dever::load('weixin/usermsg_log-insert', $update);
			}
			*/
		}
	}
	
	/**
     * 得到最终结果
     * 
     * @return mixed
     */
    private function result($key, $index, $result, $name = '')
    {
		if (isset($result[$key])) {
			$value = trim($result[$key]);
			if ($index == 'add_type') {
				$this->type($value, $result);
			}
			if ($index == 'add_content') {
				//$key = $name ? $name : $key;
				$this->result[$index][0][$key] = $value;
				$this->info($key, $value);
			} elseif ($index == 'add_ctime') {
				$this->result[$index] = $value ? $value : time();
			} else {
				$this->result[$index] = $value;
			}
		}
	}

	/**
     * 根据状态得到用户发送的具体内容，为了响应及时，这里不做解析。
     * 
     * @return mixed
     */
    private function info($key, $value)
    {
    	$this->result['add_info'][] = $value;
    }
	
	/**
     * 得到type的值
     * 
     * @return mixed
     */
    private function type($index, $result)
    {
		$type = $this->config($index);
		if (is_array($type)) {
			foreach ($type as $k => $v) {
				if (is_numeric($k)) {
					$k = $v;
				}
				$this->result($k, 'add_content', $result, $v);
			}
		} else {
			$this->result($type, 'add_content', $result);
		}
	}
	
	/**
     * 得到对应关系
     * 
     * @return mixed
     */
    private function keywords($type)
    {
		# 根据keywords的type设定来设置对应关系
		$config = array
		(
			1 => 'text',
			2 => 'news',
			//3 => 'news',
			4 => 'voice',
			5 => 'video',
			6 => 'image',
		);
		
		if (!isset($config[$type])) {
			return false;
		}
		
		$this->result['type'] = $config[$type];
		
		return $config[$type];
	}

    /**
     * 配置type类型
     * 
     * @return mixed
     */
    private function config($key, $type = 'get')
    {
    	$config = array
    	(
    		# 文本
    		'text' => array
    		(
				# 设置获取的key　应答
				'get' => 'Content',
				# 设置赋值的key 回复
				'set' => 'Content',
			),
    		# 图片
    		'image' => array
    		(
				'get' => array('PicUrl', 'MediaId'),
				'set' => array
				(
					'Image' => array('MediaId' => 'pic')
				),
			),
    		# 语音
    		'voice' => array
    		(
				'get' => array('MediaId', 'Format'),
				'set' => array
				(
					'Voice' => array('MediaId' => 'mp3')
				),
			),
			# 视频
    		'video' => array
    		(
				'get' => array('MediaId', 'ThumbMediaId'),
				'set' => array
				(
					'Voice' => array('MediaId' => 'mp4', 'Title' => 'title', 'Description' => 'info')
				),
			),
			# 小视频
    		'shortvideo' => array
    		(
				'get' => array('MediaId', 'ThumbMediaId'),
				'set' => false,
			),
			# 地理位置
    		'location' => array
    		(
				'get' => array('Location_X', 'Location_Y', 'Scale', 'Label'),
				'set' => false,
			),
			# 链接消息
    		'link' => array
    		(
				'get' => array('Title', 'Description', 'Url', 'Label'),
				'set' => false,
			),
			# 单图文消息
    		'new' => array
    		(
				'get' => false,
				'set' => array
				(
					'ArticleCount' => 1,
					'Articles' => array
					(
						'item' => array('Title' => 'title', 'Description' => 'info', 'PicUrl' => 'pic', 'Url' => 'link')
					)
				),
			),
			# 多图文消息
    		'news' => array
    		(
				'get' => false,
				'set' => array
				(
					'ArticleCount' => 'total',
					'Articles' => array
					(
						'item' => array('Title' => 'title', 'Description' => 'info', 'PicUrl' => 'pic', 'Url' => 'link')
					)
				),
			),
			# 音乐消息
			'music' => array
    		(
				'get' => false,
				'set' => false,
			),
			
			# 事件消息，所有事件返回的字段都在此做说明
			'event' => array
    		(
				'get' => array
				(
					'Event' => '事件',
					'EventKey' => '事件ID',
					'Latitude' => '纬度',
					'Longitude' => '经度',
					'Precision' => '精度',
					'Ticket' => '二维码',
					'MsgID' => '消息ID',
					'Status' => '状态',
					'TotalCount' => '粉丝数',
					'FilterCount' => '过滤数',
					'SentCount' => '成功数',
					'ErrorCount' => '失败数',
				),
				'set' => false,
			),
    	);
    	
    	return $config[$key][$type];
    }

    /**
     * 获取当前时间
     * 
     * @return mixed
     */
    private function time()
    {
		return time();
    }

    /**
     * 拼装xml相应代码
     * 
     * @return mixed
     */
    private function xml($content)
    {
    	$xml = 	"<xml>\r\n"
    	.$this->tag('ToUserName', $this->result['add_from']) . "\r\n"
    	.$this->tag('FromUserName', $this->result['add_to']) . "\r\n"
    	.$this->tag('CreateTime', $this->time()) . "\r\n"
    	.$this->tag('MsgType', $this->result['type']) . "\r\n"
    	.$content . "\r\n"
    	.'</xml>';

		return $xml;
    }

    /**
     * 生成tag
     * 
     * @return mixed
     */
    private function tag($tag, $value = '')
    {
		if (!is_numeric($value) && strpos($value, '<') === false) {
			$value = '<![CDATA['.$value.']]>';
		}
    	return '<'.$tag.'>'.$value.'</'.$tag.'>';
    } 
}